@@ -1,14 +1,23 @@
-fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str, param_name: &str) -> Option<String> {
+fn override_type_name(
+    container: &wadl::codegen::ParamContainer,
+    type_name: &str,
+    param_name: &str,
+    config: &wadl::codegen::Config,
+) -> Option<String> {
     if param_name == "entries" {
         return match container {
-            wadl::codegen::ParamContainer::Representation(rd) => {
-                match rd.id.as_deref() {
-                    Some(n) => Some(format!("Vec<{}>", map_page_to_full(wadl::codegen::camel_case_name(n).as_str()))),
-                    _ => {
-                        panic!("Unknown representation id: {}", rd.id.as_deref().unwrap_or_default());
-                    }
+            wadl::codegen::ParamContainer::Representation(rd) => match rd.id.as_deref() {
+                Some(n) => Some(format!(
+                    "Vec<{}>",
+                    map_page_to_full(wadl::codegen::camel_case_name(n).as_str())
+                )),
+                _ => {
+                    panic!(
+                        "Unknown representation id: {}",
+                        rd.id.as_deref().unwrap_or_default()
+                    );
                 }
-            }
+            },
             _ => Some("Vec<serde_json::Value>".to_string()),
         };
     }
@@ -50,7 +59,6 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "binary_package_version" => Some("debversion::Version"),
         "branch_name" => Some("String"),
         "body_text" => Some("String"),
-        "bug_reported_acknowledgement" => Some("bool"),
         "bug_reporting_guidelines" => Some("String"),
         "bug_target_display_name" => Some("String"),
         "bug_target_name" => Some("String"),
@@ -85,7 +93,13 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "custom_file_urls" => Some("Vec<url::Url>"),
         "cvs_module" => Some("String"),
         "cvs_root" => Some("String"),
-        "data" if type_name == "HostedFile" => Some("*reqwest::blocking::multipart::Part"),
+        "data" if type_name == "HostedFile" => {
+            if config.r#async {
+                Some("*reqwest::multipart::Part")
+            } else {
+                Some("*reqwest::blocking::multipart::Part")
+            }
+        }
         "deb_version_template" => Some("String"),
         "default_branch" => Some("String"),
         "default_membership_period" => Some("usize"),
@@ -93,7 +107,7 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "dependencies" => Some("Vec<String>"),
         "development_series_alias" => Some("String"),
         "diffstat" => Some("String"),
-        "display_arches" => Some("Vec<String>"),
+        "display_arches" => Some("crate::types::PackageUploadArches"),
         "display_version" => Some("String"),
         "distro_series_name" => Some("String"),
         "domain_name" => Some("String"),
@@ -110,7 +124,13 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "failnotes" => Some("String"),
         "features" => Some("Vec<String>"),
         "filename" | "file_extension" => Some("String"),
-        "file_content" => Some("*reqwest::blocking::multipart::Part"),
+        "file_content" => {
+            if config.r#async {
+                Some("*reqwest::multipart::Part")
+            } else {
+                Some("*reqwest::blocking::multipart::Part")
+            }
+        }
         "find_all_tags" => Some("bool"),
         "fingerprint" => Some("String"),
         "freshmeat_project" => Some("String"),
@@ -158,7 +178,7 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "official_answers" => Some("bool"),
         "official_blueprints" => Some("bool"),
         "official_bugs" => Some("bool"),
-        "official_bug_tags" => Some("bool"),
+        "official_bug_tags" => Some("Vec<String>"),
         "official_candidate" => Some("bool"),
         "official_codehosting" => Some("bool"),
         "official_packages" => Some("bool"),
@@ -183,13 +203,9 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "programming_lang" | "programming_language" => Some("String"),
         "project_reviewed" => Some("bool"),
         "properties" => Some("Vec<String>"),
-        "proposed_not_automatic" => Some("bool"),
         "proposition" => Some("String"),
-        "publish_by_hash" => Some("bool"),
         "qualifies_for_free_hosting" => Some("bool"),
         "recipe_text" => Some("String"),
-        "redirect_default_traversal" => Some("String"),
-        "redirect_release_uploads" => Some("bool"),
         "release_finder_url_pattern" => Some("String"),
         "release_notes" => Some("String"),
         "remote_bug" => Some("String"),
@@ -234,8 +250,6 @@ fn override_type_name(container: &wadl::codegen::ParamContainer, type_name: &str
         "suite_names" => Some("Vec<String>"),
         "summary" => Some("String"),
         "supported" => Some("bool"),
-        "supports_mirrors" => Some("bool"),
-        "supports_ppas" => Some("bool"),
         "supports_virtualized" => Some("bool"),
         "suppress_subscription_notifications" => Some("bool"),
         "tag" => Some("String"),
@@ -298,22 +312,48 @@ fn generate_representation_traits(
     _def: &wadl::ast::RepresentationDef,
     name: &str,
     _representation: &wadl::ast::RepresentationDef,
-    _config: &wadl::codegen::Config,
+    config: &wadl::codegen::Config,
 ) -> Option<Vec<String>> {
     if name.ends_with("Page") {
         let r = map_page_to_full(name);
-        let ret = vec![
-            "impl crate::page::Page for ".to_string() + name + " {\n",
-            "    type Item = ".to_string() + r.as_str() + ";\n",
-            "    fn next<'a>(&'a self, client: &'a dyn wadl::Client) -> std::result::Result<Option<Self>, Error> { self.next_collection().map(|x| x.get(client)).transpose() }\n".to_string(),
-            "    fn prev<'a>(&'a self, client: &'a dyn wadl::Client) -> std::result::Result<Option<Self>, Error> { self.prev_collection().map(|x| x.get(client)).transpose() }\n".to_string(),
+        let mut ret = vec![];
+
+        if config.r#async {
+            ret.push("#[async_trait::async_trait]\n".to_string());
+        }
+
+        ret.push(format!(
+            "impl {} for {} {{\n",
+            if config.r#async {
+                "crate::r#async::page::Page"
+            } else {
+                "crate::blocking::page::Page"
+            },
+            name
+        ));
+        ret.push("    type Item = ".to_string() + r.as_str() + ";\n");
+        ret.extend(
+            if config.r#async {
+                vec![
+            format!("    async fn next<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<Option<Self>, wadl::Error> {{ if let Some(p) = self.next_collection() {{ Ok(Some(p.get(client).await?)) }} else {{ Ok(None) }} }}\n", config.client_trait_name()),
+            format!("    async fn prev<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<Option<Self>, wadl::Error> {{ if let Some(p) = self.prev_collection() {{ Ok(Some(p.get(client).await?)) }} else {{ Ok(None) }} }}\n", config.client_trait_name()),
+                ]
+            } else {
+                vec![
+            format!("    fn next<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<Option<Self>, wadl::Error> {{ self.next_collection().map(|x| x.get(client)).transpose() }}\n", config.client_trait_name()),
+            format!("    fn prev<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<Option<Self>, wadl::Error> {{ self.prev_collection().map(|x| x.get(client)).transpose() }}\n", config.client_trait_name()),
+                ]
+            });
+
+        ret.extend(vec![
             "    fn start(&self) -> usize { self.start }\n".to_string(),
-            "    fn total_size(&self) -> Option<usize> { self.total_size.as_total_size() }\n".to_string(),
+            "    fn total_size(&self) -> Option<usize> { self.total_size.into_total_size() }\n"
+                .to_string(),
             "    fn entries(&self) -> Vec<".to_string()
                 + r.as_str()
                 + "> { self.entries.clone() }\n",
             "}\n".to_string(),
-        ];
+        ]);
         Some(ret)
     } else {
         None
@@ -344,41 +384,75 @@ fn resource_type_visibility(resource_type_name: &str) -> Option<String> {
     }
 }
 
-fn extend_accessor(param: &wadl::ast::Param, accessor_name: &str, type_name: &str, config: &wadl::codegen::Config) -> Vec<String> {
+fn extend_accessor(
+    param: &wadl::ast::Param,
+    accessor_name: &str,
+    type_name: &str,
+    config: &wadl::codegen::Config,
+) -> Vec<String> {
     // if the accessor name ends with _collection, we need to generate a more idiomatic accessor
     if let Some(field_name) = accessor_name.strip_suffix("_collection") {
         // find the bit in between the last < and the first >
-        let bn = type_name.rfind('<').map(|i| &type_name[i + 1..]).unwrap_or(type_name).trim_end_matches('>');
-        let inner_type = &bn[bn.rfind(' ').map_or(0, |x| x+1)..];
+        let bn = type_name
+            .rfind('<')
+            .map(|i| &type_name[i + 1..])
+            .unwrap_or(type_name)
+            .trim_end_matches('>');
+        let inner_type = &bn[bn.rfind(' ').map_or(0, |x| x + 1)..];
         let pr = if let Some(prefix) = inner_type.strip_suffix("PageResource") {
             prefix.to_string()
         } else {
             return vec![];
         };
         let mut lines = vec![];
-        for doc in &param.doc {
+        if let Some(doc) = param.doc.as_ref() {
             lines.extend(wadl::codegen::generate_doc(doc, 1, config));
         }
         let page_type = match pr.as_str() {
             "People" => "Person".to_string(),
-            t if t.ends_with("Countries") || t.ends_with("Repositories") || t.ends_with("Entries") => format!("{}y", t.strip_suffix("ies").unwrap()),
+            t if t.ends_with("Countries")
+                || t.ends_with("Repositories")
+                || t.ends_with("Entries") =>
+            {
+                format!("{}y", t.strip_suffix("ies").unwrap())
+            }
             t if t.ends_with("ieses") => format!("{}ies", t.strip_suffix("ieses").unwrap()),
-            "Archives" | "Bugs" | "BugTrackers" | "CharmBases" | "CharmRecipes" | "Distributions" | "Builders" | "Languages" | "Cves" | "Projects" | "Processors" | "Polls" | "Packagesets" | "Specifications" | "Snaps" | "SnapBases" | "Questions" => pr.strip_suffix('s').unwrap().to_string(),
-            t if t.ends_with("Blobs") || t.ends_with("Groups") => t.strip_suffix('s').unwrap().to_string(),
+            "Archives" | "Bugs" | "BugTrackers" | "CharmBases" | "CharmRecipes"
+            | "Distributions" | "Builders" | "Languages" | "Cves" | "Projects" | "Processors"
+            | "Polls" | "Packagesets" | "Specifications" | "Snaps" | "SnapBases" | "Questions" => {
+                pr.strip_suffix('s').unwrap().to_string()
+            }
+            t if t.ends_with("Blobs") || t.ends_with("Groups") => {
+                t.strip_suffix('s').unwrap().to_string()
+            }
             "Livefses" => "Livefs".to_string(),
             "Branches" => "Branch".to_string(),
-            t => t.to_string()
+            t => t.to_string(),
         } + "Page";
+        let pc_type = if config.r#async {
+            "crate::r#async::page::PagedCollection"
+        } else {
+            "crate::blocking::page::PagedCollection"
+        };
+        let opt_async = if config.r#async { "async " } else { "" };
         lines.extend(if type_name.starts_with("Option<") {
             vec![
-                    format!("    pub fn {}<'a>(&'a self, client: &'a dyn wadl::Client) -> std::result::Result<Option<crate::page::PagedCollection<'a, {}>>, Error> {{\n", field_name, page_type),
-                    format!("        self.{}_collection().map(|x| Ok(crate::page::PagedCollection::new(client, x.get(client)?))).transpose()\n", field_name),
+                    format!("    pub {}fn {}<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<Option<{}<'a, {}>>, wadl::Error> {{\n", opt_async, field_name, config.client_trait_name(), pc_type, page_type),
+                    if config.r#async {
+                        format!("        if let Some(c) = self.{}_collection() {{ Ok(Some({}::new(client, c.get(client).await?))) }} else {{ Ok(None) }}\n", field_name, pc_type)
+                    } else {
+                        format!("        self.{}_collection().map(|x| Ok({}::new(client, x.get(client)?))).transpose()\n", field_name, pc_type)
+                    },
                     format!("    }}\n"),
             ]
         } else {
             vec![
-                format!("    pub fn {}<'a>(&'a self, client: &'a dyn wadl::Client) -> std::result::Result<crate::page::PagedCollection<'a, {}>, Error> {{\n", field_name, page_type),
-                format!("        Ok(crate::page::PagedCollection::new(client, self.{}_collection().get(client)?))\n", field_name),
+                format!("    pub {}fn {}<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<{}<'a, {}>, wadl::Error> {{\n", opt_async, field_name, config.client_trait_name(), pc_type, page_type),
+                if config.r#async {
+                    format!("        Ok({}::new(client, self.{}_collection().get(client).await?))\n", pc_type, field_name)
+                } else {
+                    format!("        Ok({}::new(client, self.{}_collection().get(client)?))\n", pc_type, field_name)
+                },
                 format!("    }}\n"),
             ]
         });
@@ -388,12 +462,26 @@ fn extend_accessor(param: &wadl::ast::Param, accessor_name: &str, type_name: &st
     }
 }
 
-fn extend_method(resource_type: &str, name: &str, ret_type: &str, _config: &wadl::codegen::Config) -> Vec<String> {
+fn extend_method(
+    resource_type: &str,
+    name: &str,
+    ret_type: &str,
+    config: &wadl::codegen::Config,
+) -> Vec<String> {
+    let pc_type = if config.r#async {
+        "crate::r#async::page::PagedCollection"
+    } else {
+        "crate::blocking::page::PagedCollection"
+    };
     if !resource_type.ends_with("-page-resource") && name == "get" && ret_type.contains("Page") {
         vec![
             format!("    /// Get a paged collection of {}.\n", ret_type),
-            format!("    pub fn iter<'a>(&'a self, client: &'a dyn wadl::Client) -> std::result::Result<crate::page::PagedCollection<'a, {}>, Error> {{\n", ret_type),
-            format!("        Ok(crate::page::PagedCollection::new(client, self.get(client)?))\n"),
+            format!("    pub {}fn iter<'a>(&'a self, client: &'a dyn {}) -> std::result::Result<{}<'a, {}>, wadl::Error> {{\n", if config.r#async { "async " } else { "" }, config.client_trait_name(), pc_type, ret_type),
+            if config.r#async {
+                format!("        Ok({}::new(client, self.get(client).await?))\n", pc_type)
+            } else {
+                format!("        Ok({}::new(client, self.get(client)?))\n", pc_type)
+            },
             format!("    }}\n"),
         ]
     } else {
@@ -401,7 +489,11 @@ fn extend_method(resource_type: &str, name: &str, ret_type: &str, _config: &wadl
     }
 }
 
-fn map_type_for_response(method: &str, type_name: &str) -> Option<(String, String)> {
+fn map_type_for_response(
+    method: &str,
+    type_name: &str,
+    config: &wadl::codegen::Config,
+) -> Option<(String, String)> {
     if !type_name.ends_with("Page") {
         return None;
     }
@@ -410,7 +502,16 @@ fn map_type_for_response(method: &str, type_name: &str) -> Option<(String, Strin
         return None;
     }
 
-    Some((format!("crate::page::PagedCollection<'a, {}>", type_name), "|x| crate::page::PagedCollection::new(client, x)".to_string()))
+    let pc_type = if config.r#async {
+        "crate::r#async::page::PagedCollection"
+    } else {
+        "crate::blocking::page::PagedCollection"
+    };
+
+    Some((
+        format!("{}<'a, {}>", pc_type, type_name),
+        format!("|x| {}::new(client, x)", pc_type),
+    ))
 }
 
 fn deprecated_param(param: &wadl::ast::Param) -> bool {
@@ -422,48 +523,96 @@ fn deprecated_param(param: &wadl::ast::Param) -> bool {
 }
 
 fn options_enum_name(param: &wadl::ast::Param, exists: Box<dyn Fn(&str) -> bool>) -> String {
-    let options = param.options.as_ref().unwrap().keys().collect::<std::collections::HashSet<_>>();
+    let options = param
+        .options
+        .as_ref()
+        .unwrap()
+        .keys()
+        .collect::<std::collections::HashSet<_>>();
     let name = match param.name.as_str() {
-        "status" => {
-            match param.doc.as_ref().unwrap().content.as_str().trim() {
-                n if n.contains("The new status of the merge proposal.") => "MergeProposalStatus".to_string(),
-                n if n.contains("The status of this publishing record") => "PublishingRecordStatus".to_string(),
-                n if n.contains("Return only items that have this status.") => "PackageUploadStatus".to_string(),
-                n if n.contains("The state of this membership") => "TeamMembershipStatus".to_string(),
-                n if n.contains("The status of this subscription") => "ArchiveSubscriptionStatus".to_string(),
-                _ if options == ["Nominated", "Approved", "Declined"].into_iter().collect::<std::collections::HashSet<_>>() => "BugNominationStatus".to_string(),
-                _ if options == ["Completed", "Pending", "Failed"].into_iter().collect::<std::collections::HashSet<_>>() => "CharmRecipeStatus".to_string(),
-                _ if options.contains("Won't Fix") => "BugTaskStatus".to_string(),
-                n if n.contains("Whether or not the vulnerability has been reviewed and") => "CveStatus".to_string(),
-                n if n.contains("The current status of a mirror") => "MirrorStatus".to_string(),
-                n if n.contains("The current status of this difference") => "DistroSeriesDifferenceStatus".to_string(),
-                _ => {
-                    let mut name = "Status".to_string();
-                    while exists(name.as_str()) {
-                        name.push('_');
-                    }
-                    name
-                }
+        "status" => match param.doc.as_ref().unwrap().content.as_str().trim() {
+            n if n.contains("The new status of the merge proposal.") => {
+                "MergeProposalStatus".to_string()
             }
-        }
+            n if n.contains("The status of this publishing record") => {
+                "PublishingRecordStatus".to_string()
+            }
+            n if n.contains("Return only items that have this status.") => {
+                "PackageUploadStatus".to_string()
+            }
+            n if n.contains("The state of this membership") => "TeamMembershipStatus".to_string(),
+            n if n.contains("The status of this subscription") => {
+                "ArchiveSubscriptionStatus".to_string()
+            }
+            _ if options
+                == ["Nominated", "Approved", "Declined"]
+                    .into_iter()
+                    .collect::<std::collections::HashSet<_>>() =>
+            {
+                "BugNominationStatus".to_string()
+            }
+            _ if options
+                == ["Completed", "Pending", "Failed"]
+                    .into_iter()
+                    .collect::<std::collections::HashSet<_>>() =>
+            {
+                "CharmRecipeStatus".to_string()
+            }
+            _ if options.contains("Won't Fix") => "BugTaskStatus".to_string(),
+            n if n.contains("Whether or not the vulnerability has been reviewed and") => {
+                "CveStatus".to_string()
+            }
+            n if n.contains("The current status of a mirror") => "MirrorStatus".to_string(),
+            n if n.contains("The current status of this difference") => {
+                "DistroSeriesDifferenceStatus".to_string()
+            }
+            _ => {
+                let mut name = "Status".to_string();
+                while exists(name.as_str()) {
+                    name.push('_');
+                }
+                name
+            }
+        },
         "lifecycle_status" => {
             if param.doc.as_ref().unwrap().content.as_str().contains("Cve") {
                 "CveLifecycleStatus".to_string()
-            } else if options == ["Experimental", "Development", "Mature", "Merged", "Abandoned"].into_iter().collect::<std::collections::HashSet<_>>() {
+            } else if options
+                == [
+                    "Experimental",
+                    "Development",
+                    "Mature",
+                    "Merged",
+                    "Abandoned",
+                ]
+                .into_iter()
+                .collect::<std::collections::HashSet<_>>()
+            {
                 "BranchLifecycleStatus".to_string()
-            } else if options == ["Started", "Not started", "Complete"].into_iter().collect::<std::collections::HashSet<_>>() {
+            } else if options
+                == ["Started", "Not started", "Complete"]
+                    .into_iter()
+                    .collect::<std::collections::HashSet<_>>()
+            {
                 "SpecificationLifecycleStatus".to_string()
             } else {
                 panic!("Unknown lifecycle_status options: {:?}", options);
             }
-        },
+        }
         "type" => {
-            if param.doc.as_ref().unwrap().content.as_str().contains("Attachment Type") {
+            if param
+                .doc
+                .as_ref()
+                .unwrap()
+                .content
+                .as_str()
+                .contains("Attachment Type")
+            {
                 "AttachmentType".to_string()
             } else {
                 "Type".to_string()
             }
-        },
+        }
         "order_by" => {
             if options.contains("by branch name") {
                 "BranchOrderBy".to_string()
@@ -474,7 +623,7 @@ fn options_enum_name(param: &wadl::ast::Param, exists: Box<dyn Fn(&str) -> bool>
             } else {
                 panic!("Unknown order_by options: {:?}", options);
             }
-        },
+        }
         "repository_format" => {
             if options.iter().any(|o| o.contains("Bazaar")) {
                 "BazaarRepositoryFormat".to_string()
@@ -482,7 +631,7 @@ fn options_enum_name(param: &wadl::ast::Param, exists: Box<dyn Fn(&str) -> bool>
                 "ArchiveRepositoryFormat".to_string()
             }
         }
-        n => wadl::codegen::camel_case_name(n)
+        n => wadl::codegen::camel_case_name(n),
     };
 
     name
@@ -493,8 +642,14 @@ fn reformat_docstring(text: &str) -> String {
 }
 
 fn convert_to_multipart(type_name: &str, expr: &str) -> Option<String> {
-    let inner_type = type_name.trim_start_matches("Vec<").trim_end_matches('>').trim_start_matches("Option<").trim_end_matches('>');
-    if inner_type == "reqwest::blocking::multipart::Part" {
+    let inner_type = type_name
+        .trim_start_matches("Vec<")
+        .trim_end_matches('>')
+        .trim_start_matches("Option<")
+        .trim_end_matches('>');
+    if inner_type == "reqwest::blocking::multipart::Part"
+        || inner_type == "reqwest::multipart::Part"
+    {
         Some(expr.replace('&', "").replace(".url().to_string()", ""))
     } else {
         None
@@ -504,53 +659,58 @@ fn convert_to_multipart(type_name: &str, expr: &str) -> Option<String> {
 const VERSIONS: &[&str] = &["1.0", "devel", "beta"];
 
 fn main() {
-    #[allow(clippy::needless_update)]
-    let config = wadl::codegen::Config {
-        override_type_name: Some(Box::new(override_type_name)),
-        param_accessor_rename: Some(Box::new(accessor_rename)),
-        generate_representation_traits: Some(Box::new(generate_representation_traits)),
-        strip_code_examples: true,
-        accessor_visibility: Some(Box::new(accessor_visibility)),
-        resource_type_visibility: Some(Box::new(resource_type_visibility)),
-        extend_accessor: Some(Box::new(extend_accessor)),
-        extend_method: Some(Box::new(extend_method)),
-        method_visibility: Some(Box::new(method_visibility)),
-        map_type_for_response: Some(Box::new(map_type_for_response)),
-        deprecated_param: Some(Box::new(deprecated_param)),
-        options_enum_name: Some(Box::new(options_enum_name)),
-        reformat_docstring: Some(Box::new(reformat_docstring)),
-        convert_to_multipart: Some(Box::new(convert_to_multipart)),
-        ..Default::default()
-    };
-
-    for version in VERSIONS {
-        let wadl = if let Ok(text) = std::fs::read_to_string(format!(
-            "{}/wadl/{}.wadl",
-            env!("CARGO_MANIFEST_DIR"),
-            version
-        )) {
-            text
-        } else {
-            let url = format!("https://api.launchpad.net/{}/", version);
-            reqwest::blocking::Client::new()
-                .request(reqwest::Method::GET, &url)
-                .header("Accept", "application/vd.sun.wadl+xml")
-                .send()
-                .unwrap()
-                .error_for_status()
-                .unwrap()
-                .text()
-                .unwrap()
+    for is_async in [true, false] {
+        let config = wadl::codegen::Config {
+            override_type_name: Some(Box::new(override_type_name)),
+            param_accessor_rename: Some(Box::new(accessor_rename)),
+            generate_representation_traits: Some(Box::new(generate_representation_traits)),
+            strip_code_examples: true,
+            accessor_visibility: Some(Box::new(accessor_visibility)),
+            resource_type_visibility: Some(Box::new(resource_type_visibility)),
+            extend_accessor: Some(Box::new(extend_accessor)),
+            extend_method: Some(Box::new(extend_method)),
+            method_visibility: Some(Box::new(method_visibility)),
+            map_type_for_response: Some(Box::new(map_type_for_response)),
+            deprecated_param: Some(Box::new(deprecated_param)),
+            options_enum_name: Some(Box::new(options_enum_name)),
+            reformat_docstring: Some(Box::new(reformat_docstring)),
+            convert_to_multipart: Some(Box::new(convert_to_multipart)),
+            r#async: is_async,
+            ..Default::default()
         };
 
-        let wadl_app = wadl::parse_string(wadl.as_str()).unwrap();
-        let code = wadl::codegen::generate(&wadl_app, &config);
-        let target_dir = std::path::PathBuf::from(std::env::var("OUT_DIR").unwrap())
-            .canonicalize()
-            .unwrap();
-        let generated = target_dir.join("generated");
-        std::fs::create_dir_all(&generated).unwrap();
-        let path = generated.join(format!("{}.rs", version.replace('.', "_")));
-        std::fs::write(path, code).unwrap();
+        for version in VERSIONS {
+            let wadl = if let Ok(text) = std::fs::read_to_string(format!(
+                "{}/wadl/{}.wadl",
+                env!("CARGO_MANIFEST_DIR"),
+                version
+            )) {
+                text
+            } else {
+                let url = format!("https://api.launchpad.net/{}/", version);
+                reqwest::blocking::Client::new()
+                    .request(reqwest::Method::GET, &url)
+                    .header("Accept", "application/vd.sun.wadl+xml")
+                    .send()
+                    .unwrap()
+                    .error_for_status()
+                    .unwrap()
+                    .text()
+                    .unwrap()
+            };
+
+            let wadl_app = wadl::parse_string(wadl.as_str()).unwrap();
+            let code = wadl::codegen::generate(&wadl_app, &config);
+            let target_dir = std::path::PathBuf::from(std::env::var("OUT_DIR").unwrap())
+                .canonicalize()
+                .unwrap();
+            let mut generated = target_dir.join("generated");
+            if is_async {
+                generated = generated.join("async");
+            }
+            std::fs::create_dir_all(&generated).unwrap();
+            let path = generated.join(format!("{}.rs", version.replace('.', "_")));
+            std::fs::write(path, code).unwrap();
+        }
     }
 }
